#!/usr/bin/python
#encoding=utf-8
import DescribeBackupsRequest
import json
import os
import urllib2
from optparse import OptionParser
from aliyunsdkcore.client import AcsClient
from datetime import datetime

MYDIR = os.path.dirname(__file__)
RDB_FILE_PATH = "/root/"

def download_remote_file(url, local_file_path):
    f = urllib2.urlopen(url)
    data = f.read()
    with open(local_file_path, "wb") as code:
        code.write(data)
    return local_file_path

def getBackup(instanceId, startTime, backupId=None,client=None):
    request = DescribeBackupsRequest.DescribeBackupsRequest()
    # client._url_test_flag = True
    # request.set_InstanceId('r-m5ec7db53da93674')
    request.set_InstanceId(instanceId)
    request.set_StartTime("%sT00:01Z"%startTime)
    request.set_EndTime("%sT23:59Z"%startTime)
    if backupId:
        request.set_BackupId(backupId)
    response = json.loads(client.do_action_with_exception(request))
    print(response)
    if "Backups" in response and "Backup" in response["Backups"] and len(response["Backups"]["Backup"]) > 0:
        print(response["Backups"]["Backup"])
        # url = "https://rdsbak-qd-v2.oss-cn-qingdao.aliyuncs.com/custins7155229/hins4922443_data_20180718054827.rdb?OSSAccessKeyId=LTAITfQ7krsrEwRn&Expires=1531979093&Signature=Zh9fik7WaqBcHbkpyMO8tHToTa8%3D"
        return response["Backups"]["Backup"]
    raise Exception("Exception:getBackup is fail and response:[%s]"%response)

def main():
    usage = """usage: %prog [options] 
        Example : python %prog -c r-m5ec7db53da93674 -d 2018-07-17 -t 
        rm-davinx-003.redis.6d6e3611-8.newtest.rdstest.aliyun-inc.com -p 6379 -a Davinx123456 """

    parser = OptionParser(usage=usage)
    parser.add_option("-c", "--instance_id", dest="instance_id",
                      help="instance_id is instance id")
    parser.add_option("-o", "--output", dest="output",
                      help="Output rdb file and default path /root/", metavar="FILE")
    parser.add_option("-b", "--bakid", dest="bakid", help="instance backup id")
    parser.add_option("-d", "--day", dest="day", help="backup day")
    parser.add_option("-t", "--target", dest="target", help="target instance ip")
    parser.add_option("-p", "--port", dest="port", help="target instance port")
    parser.add_option("-a", "--auth", dest="auth", help="target instance auth password")
    (options, args) = parser.parse_args()
    print options
    filters = {}
    if options.instance_id:
        filters['custins_id'] = options.instance_id
    else:
        parser.error("Redis instance_id not specified")
    filters['rdb_file_path'] = RDB_FILE_PATH
    if options.output:
        filters['rdb_file_path'] = options.output
    # filters['bakid'] = "256199521"
    filters['bakid'] = None
    if options.bakid:
        filters['bakid'] = options.bakid
    if options.day:
        filters['start_time'] = options.day
    else:
        now = datetime.now()
        filters['start_time'] = "%s-%s-%s"%(now.year, now.month, now.day)
    if options.target:
        filters['ip'] = options.target
    else:
        parser.error("target instance ip not specified")
    if options.port:
        filters['port'] = options.port
    else:
        parser.error("target instance port not specified")
    if options.auth:
        filters['auth'] = options.auth
    else:
        parser.error("target instance auth password not specified")
    config = {}
    with open(os.path.join(MYDIR, "config.json"), 'r') as load_f:
        config = json.load(load_f)
    client = AcsClient(
        config["acesskeyID"],
        config["acesskeySecret"],
        config["regionid"])
    if filters['bakid']:
        backupIds = filters['bakid'].split(",")
        for backupId in backupIds:
            backups = getBackup(filters['custins_id'], filters['start_time'], backupId, client)
            for bak in backups:
                instance_id = bak["NodeInstanceId"]
                rdb_file_name = "%s%s.rdb" % (filters['rdb_file_path'], instance_id)
                ##download remote rdb file to local
                rdb_file = download_remote_file(bak["BackupDownloadURL"], rdb_file_name)
                # rdb_file = download_remote_file(bak["BackupIntranetDownloadURL"], rdb_file_name)
                ##恢复到目标实例
                p = os.system("chmod +x redis-port")
                p = os.system("./redis-port restore --input=%s --target=%s:%s "
                              "--auth=%s --logfile=logs/redis-port.log" % (rdb_file, filters['ip'],filters['port'],filters['auth']))
                print(p)
    else:
        backups = getBackup(filters['custins_id'], filters['start_time'], filters['bakid'], client)
        for bak in backups:
            instance_id = bak["NodeInstanceId"]
            rdb_file_name = "%s%s.rdb" % (filters['rdb_file_path'], instance_id)
            ##download remote rdb file to local
            rdb_file = download_remote_file(bak["BackupDownloadURL"], rdb_file_name)
            # rdb_file = download_remote_file(bak["BackupIntranetDownloadURL"], rdb_file_name)
            ##恢复到目标实例
            p = os.system("chmod +x redis-port")
            p = os.system("./redis-port restore --input=%s --target=%s:%s "
                          "--auth=%s --logfile=logs/redis-port.log" % (
                          rdb_file, filters['ip'], filters['port'], filters['auth']))
            print(p)

if __name__ == '__main__':
    main()
    pass